@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2007,2008,2009 Intel Corporation.
+ * Copyright (C) 2007,2008,2009,2013 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -95,6 +95,9 @@ _cogl_init_feature_overrides (CoglContext *ctx)
     {
       ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
       COGL_FLAGS_SET (ctx->features, COGL_FEATURE_ID_GLSL, FALSE);
+      COGL_FLAGS_SET (ctx->features,
+                      COGL_FEATURE_ID_PER_VERTEX_POINT_SIZE,
+                      FALSE);
     }
 
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_NPOT_TEXTURES)))
@@ -133,7 +136,7 @@ cogl_context_new (CoglDisplay *display,
                   CoglError **error)
 {
   CoglContext *context;
-  GLubyte default_texture_data[] = { 0xff, 0xff, 0xff, 0x0 };
+  uint8_t default_texture_data[] = { 0xff, 0xff, 0xff, 0xff };
   CoglBitmap *default_texture_bitmap;
   const CoglWinsysVtable *winsys;
   int i;
@@ -312,7 +315,8 @@ cogl_context_new (CoglDisplay *display,
   context->swap_callback_closures =
     g_hash_table_new (g_direct_hash, g_direct_equal);
 
-  COGL_TAILQ_INIT (&context->onscreen_events_queue);
+  _cogl_list_init (&context->onscreen_events_queue);
+  _cogl_list_init (&context->onscreen_dirty_queue);
 
   g_queue_init (&context->gles2_context_stack);
 
@@ -324,7 +328,7 @@ cogl_context_new (CoglDisplay *display,
 
   context->current_pipeline = NULL;
   context->current_pipeline_changes_since_flush = 0;
-  context->current_pipeline_skip_gl_color = FALSE;
+  context->current_pipeline_with_color_attrib = FALSE;
 
   _cogl_bitmask_init (&context->enabled_builtin_attributes);
   _cogl_bitmask_init (&context->enable_builtin_attributes_tmp);
@@ -477,6 +481,8 @@ cogl_context_new (CoglDisplay *display,
       cogl_has_feature (context, COGL_FEATURE_ID_POINT_SPRITE))
     GE (context, glEnable (GL_POINT_SPRITE));
 
+  _cogl_list_init (&context->fences);
+
   return context;
 }
 
@@ -607,6 +613,12 @@ cogl_context_get_display (CoglContext *context)
   return context->display;
 }
 
+CoglRenderer *
+cogl_context_get_renderer (CoglContext *context)
+{
+  return context->display->renderer;
+}
+
 #ifdef COGL_HAS_EGL_SUPPORT
 EGLDisplay
 cogl_egl_context_get_egl_display (CoglContext *context)
