@@ -170,6 +170,10 @@ struct _CoglFramebuffer
   int                 clear_clip_y1;
   CoglBool            clear_clip_dirty;
 
+  /* Whether something has been drawn to the buffer since the last
+   * swap buffers or swap region. */
+  CoglBool            mid_scene;
+
   /* driver specific */
   CoglBool            dirty_bitmasks;
   CoglFramebufferBits bits;
@@ -233,7 +237,7 @@ _cogl_framebuffer_clear_without_flush4f (CoglFramebuffer *framebuffer,
                                          float alpha);
 
 void
-_cogl_framebuffer_dirty (CoglFramebuffer *framebuffer);
+_cogl_framebuffer_mark_mid_scene (CoglFramebuffer *framebuffer);
 
 CoglClipState *
 _cogl_framebuffer_get_clip_state (CoglFramebuffer *framebuffer);
