@@ -27,14 +27,17 @@
 #ifndef __COGL_MATRIX_H
 #define __COGL_MATRIX_H
 
+#include <cogl/cogl-defines.h>
+
 #ifdef COGL_HAS_GTYPE_SUPPORT
 #include <glib-object.h>
 #endif /* COGL_HAS_GTYPE_SUPPORT */
 
-#include "cogl-types.h"
+#include <cogl/cogl-types.h>
+#include <cogl/cogl-macros.h>
 
 #ifdef COGL_ENABLE_EXPERIMENTAL_API
-#include "cogl-quaternion.h"
+#include <cogl/cogl-quaternion.h>
 #endif
 
 COGL_BEGIN_DECLS
@@ -408,7 +411,8 @@ cogl_matrix_ortho (CoglMatrix *matrix,
                    float       bottom,
                    float       top,
                    float       near,
-                   float       far);
+                   float       far)
+     COGL_DEPRECATED_IN_1_10_FOR (cogl_matrix_orthographic);
 
 #ifdef COGL_ENABLE_EXPERIMENTAL_API
 /**
