@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2011 Intel Corporation.
+ * Copyright (C) 2011,2013 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,8 @@
 
 #include "cogl-onscreen.h"
 #include "cogl-framebuffer-private.h"
-#include "cogl-queue.h"
+#include "cogl-closure-list-private.h"
+#include "cogl-list.h"
 
 #include <glib.h>
 
@@ -34,43 +35,22 @@
 #include <windows.h>
 #endif
 
-COGL_TAILQ_HEAD (CoglFrameCallbackList, CoglFrameClosure);
-
-struct _CoglFrameClosure
+typedef struct _CoglOnscreenEvent
 {
-  COGL_TAILQ_ENTRY (CoglFrameClosure) list_node;
-
-  CoglFrameCallback callback;
-
-  void *user_data;
-  CoglUserDataDestroyCallback destroy;
-};
-
-typedef struct _CoglResizeNotifyEntry CoglResizeNotifyEntry;
-
-COGL_TAILQ_HEAD (CoglResizeNotifyList, CoglResizeNotifyEntry);
-
-struct _CoglResizeNotifyEntry
-{
-  COGL_TAILQ_ENTRY (CoglResizeNotifyEntry) list_node;
-
-  CoglOnscreenResizeCallback callback;
-  void *user_data;
-  unsigned int id;
-};
-
-typedef struct _CoglOnscreenEvent CoglOnscreenEvent;
-
-COGL_TAILQ_HEAD (CoglOnscreenEventList, CoglOnscreenEvent);
-
-struct _CoglOnscreenEvent
-{
-  COGL_TAILQ_ENTRY (CoglOnscreenEvent) list_node;
+  CoglList link;
 
   CoglOnscreen *onscreen;
   CoglFrameInfo *info;
   CoglFrameEvent type;
-};
+} CoglOnscreenEvent;
+
+typedef struct _CoglOnscreenQueuedDirty
+{
+  CoglList link;
+
+  CoglOnscreen *onscreen;
+  CoglOnscreenDirtyInfo info;
+} CoglOnscreenQueuedDirty;
 
 struct _CoglOnscreen
 {
@@ -86,12 +66,18 @@ struct _CoglOnscreen
   HWND foreign_hwnd;
 #endif
 
+#ifdef COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT
+  struct wl_surface *foreign_surface;
+#endif
+
   CoglBool swap_throttled;
 
-  CoglFrameCallbackList frame_closures;
+  CoglList frame_closures;
 
   CoglBool resizable;
-  CoglResizeNotifyList resize_callbacks;
+  CoglList resize_closures;
+
+  CoglList dirty_closures;
 
   int64_t frame_counter;
   int64_t swap_frame_counter; /* frame counter at last all to
@@ -119,6 +105,11 @@ void
 _cogl_onscreen_notify_resize (CoglOnscreen *onscreen);
 
 void
-_cogl_dispatch_onscreen_events (CoglContext *context);
+_cogl_onscreen_queue_dirty (CoglOnscreen *onscreen,
+                            const CoglOnscreenDirtyInfo *info);
+
+
+void
+_cogl_onscreen_queue_full_dirty (CoglOnscreen *onscreen);
 
 #endif /* __COGL_ONSCREEN_PRIVATE_H */
