@@ -33,6 +33,8 @@
 #include "cogl-context-private.h"
 #include "cogl-object-private.h"
 #include "cogl1-context.h"
+#include "cogl-closure-list-private.h"
+#include "cogl-poll-private.h"
 
 static void _cogl_onscreen_free (CoglOnscreen *onscreen);
 
@@ -46,8 +48,9 @@ _cogl_onscreen_init_from_template (CoglOnscreen *onscreen,
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
 
-  COGL_TAILQ_INIT (&onscreen->frame_closures);
-  COGL_TAILQ_INIT (&onscreen->resize_callbacks);
+  _cogl_list_init (&onscreen->frame_closures);
+  _cogl_list_init (&onscreen->resize_closures);
+  _cogl_list_init (&onscreen->dirty_closures);
 
   framebuffer->config = onscreen_template->config;
   cogl_object_ref (framebuffer->config.swap_chain);
@@ -115,25 +118,11 @@ _cogl_onscreen_free (CoglOnscreen *onscreen)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   const CoglWinsysVtable *winsys = _cogl_framebuffer_get_winsys (framebuffer);
-  CoglResizeNotifyEntry *resize_entry;
-  CoglFrameClosure *frame_closure;
   CoglFrameInfo *frame_info;
 
-  while ((resize_entry = COGL_TAILQ_FIRST (&onscreen->resize_callbacks)))
-    {
-      COGL_TAILQ_REMOVE (&onscreen->resize_callbacks, resize_entry, list_node);
-      g_slice_free (CoglResizeNotifyEntry, resize_entry);
-    }
-
-  while ((frame_closure = COGL_TAILQ_FIRST (&onscreen->frame_closures)))
-    {
-      COGL_TAILQ_REMOVE (&onscreen->frame_closures, frame_closure, list_node);
-
-      if (frame_closure->destroy)
-        frame_closure->destroy (frame_closure->user_data);
-
-      g_slice_free (CoglFrameClosure, frame_closure);
-    }
+  _cogl_closure_list_disconnect_all (&onscreen->resize_closures);
+  _cogl_closure_list_disconnect_all (&onscreen->frame_closures);
+  _cogl_closure_list_disconnect_all (&onscreen->dirty_closures);
 
   while ((frame_info = g_queue_pop_tail (&onscreen->pending_frame_infos)))
     cogl_object_unref (frame_info);
@@ -151,6 +140,108 @@ _cogl_onscreen_free (CoglOnscreen *onscreen)
   g_free (onscreen);
 }
 
+static void
+notify_event (CoglOnscreen *onscreen,
+              CoglFrameEvent event,
+              CoglFrameInfo *info)
+{
+  _cogl_closure_list_invoke (&onscreen->frame_closures,
+                             CoglFrameCallback,
+                             onscreen, event, info);
+}
+
+static void
+_cogl_dispatch_onscreen_cb (CoglContext *context)
+{
+  CoglOnscreenEvent *event, *tmp;
+  CoglList queue;
+
+  /* Dispatching the event callback may cause another frame to be
+   * drawn which in may cause another event to be queued immediately.
+   * To make sure this loop will only dispatch one set of events we'll
+   * steal the queue and iterate that separately */
+  _cogl_list_init (&queue);
+  _cogl_list_insert_list (&queue, &context->onscreen_events_queue);
+  _cogl_list_init (&context->onscreen_events_queue);
+
+  _cogl_closure_disconnect (context->onscreen_dispatch_idle);
+  context->onscreen_dispatch_idle = NULL;
+
+  _cogl_list_for_each_safe (event, tmp, &queue, link)
+    {
+      CoglOnscreen *onscreen = event->onscreen;
+      CoglFrameInfo *info = event->info;
+
+      notify_event (onscreen, event->type, info);
+
+      cogl_object_unref (onscreen);
+      cogl_object_unref (info);
+
+      g_slice_free (CoglOnscreenEvent, event);
+    }
+
+  while (!_cogl_list_empty (&context->onscreen_dirty_queue))
+    {
+      CoglOnscreenQueuedDirty *qe =
+        _cogl_container_of (context->onscreen_dirty_queue.next, qe, link);
+
+      _cogl_list_remove (&qe->link);
+
+      _cogl_closure_list_invoke (&qe->onscreen->dirty_closures,
+                                 CoglOnscreenDirtyCallback,
+                                 qe->onscreen,
+                                 &qe->info);
+
+      cogl_object_unref (qe->onscreen);
+
+      g_slice_free (CoglOnscreenQueuedDirty, qe);
+    }
+}
+
+static void
+_cogl_onscreen_queue_dispatch_idle (CoglOnscreen *onscreen)
+{
+  CoglContext *ctx = COGL_FRAMEBUFFER (onscreen)->context;
+
+  if (!ctx->onscreen_dispatch_idle)
+    {
+      ctx->onscreen_dispatch_idle =
+        _cogl_poll_renderer_add_idle (ctx->display->renderer,
+                                      (CoglIdleCallback)
+                                      _cogl_dispatch_onscreen_cb,
+                                      ctx,
+                                      NULL);
+    }
+}
+
+void
+_cogl_onscreen_queue_dirty (CoglOnscreen *onscreen,
+                            const CoglOnscreenDirtyInfo *info)
+{
+  CoglContext *ctx = COGL_FRAMEBUFFER (onscreen)->context;
+  CoglOnscreenQueuedDirty *qe = g_slice_new (CoglOnscreenQueuedDirty);
+
+  qe->onscreen = cogl_object_ref (onscreen);
+  qe->info = *info;
+  _cogl_list_insert (ctx->onscreen_dirty_queue.prev, &qe->link);
+
+  _cogl_onscreen_queue_dispatch_idle (onscreen);
+}
+
+void
+_cogl_onscreen_queue_full_dirty (CoglOnscreen *onscreen)
+{
+  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
+  CoglOnscreenDirtyInfo info;
+
+  info.x = 0;
+  info.y = 0;
+  info.width = framebuffer->width;
+  info.height = framebuffer->height;
+
+  _cogl_onscreen_queue_dirty (onscreen, &info);
+}
+
 static void
 _cogl_onscreen_queue_event (CoglOnscreen *onscreen,
                             CoglFrameEvent type,
@@ -164,11 +255,15 @@ _cogl_onscreen_queue_event (CoglOnscreen *onscreen,
   event->info = cogl_object_ref (info);
   event->type = type;
 
-  COGL_TAILQ_INSERT_TAIL (&ctx->onscreen_events_queue, event, list_node);
+  _cogl_list_insert (ctx->onscreen_events_queue.prev, &event->link);
+
+  _cogl_onscreen_queue_dispatch_idle (onscreen);
 }
 
 void
-cogl_onscreen_swap_buffers (CoglOnscreen *onscreen)
+cogl_onscreen_swap_buffers_with_damage (CoglOnscreen *onscreen,
+                                        const int *rectangles,
+                                        int n_rectangles)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   const CoglWinsysVtable *winsys;
@@ -184,7 +279,8 @@ cogl_onscreen_swap_buffers (CoglOnscreen *onscreen)
   cogl_flush ();
 
   winsys = _cogl_framebuffer_get_winsys (framebuffer);
-  winsys->onscreen_swap_buffers (COGL_ONSCREEN (framebuffer));
+  winsys->onscreen_swap_buffers_with_damage (onscreen,
+                                             rectangles, n_rectangles);
   cogl_framebuffer_discard_buffers (framebuffer,
                                     COGL_BUFFER_BIT_COLOR |
                                     COGL_BUFFER_BIT_DEPTH |
@@ -205,6 +301,13 @@ cogl_onscreen_swap_buffers (CoglOnscreen *onscreen)
     }
 
   onscreen->frame_counter++;
+  framebuffer->mid_scene = FALSE;
+}
+
+void
+cogl_onscreen_swap_buffers (CoglOnscreen *onscreen)
+{
+  cogl_onscreen_swap_buffers_with_damage (onscreen, NULL, 0);
 }
 
 void
@@ -255,6 +358,7 @@ cogl_onscreen_swap_region (CoglOnscreen *onscreen,
     }
 
   onscreen->frame_counter++;
+  framebuffer->mid_scene = FALSE;
 }
 
 int
@@ -361,15 +465,10 @@ cogl_onscreen_add_frame_callback (CoglOnscreen *onscreen,
                                   void *user_data,
                                   CoglUserDataDestroyCallback destroy)
 {
-  CoglFrameClosure *closure = g_slice_new0 (CoglFrameClosure);
-
-  closure->callback = callback;
-  closure->user_data = user_data;
-  closure->destroy = destroy;
-
-  COGL_TAILQ_INSERT_TAIL (&onscreen->frame_closures, closure, list_node);
-
-  return closure;
+  return _cogl_closure_list_add (&onscreen->frame_closures,
+                                 callback,
+                                 user_data,
+                                 destroy);
 }
 
 void
@@ -378,12 +477,7 @@ cogl_onscreen_remove_frame_callback (CoglOnscreen *onscreen,
 {
   _COGL_RETURN_IF_FAIL (closure);
 
-  if (closure->destroy)
-    closure->destroy (closure->user_data);
-
-  COGL_TAILQ_REMOVE (&onscreen->frame_closures, closure, list_node);
-
-  g_slice_free (CoglFrameClosure, closure);
+  _cogl_closure_disconnect (closure);
 }
 
 typedef struct _SwapBufferCallbackState
@@ -503,54 +597,6 @@ cogl_onscreen_hide (CoglOnscreen *onscreen)
     }
 }
 
-static void
-notify_event (CoglOnscreen *onscreen,
-              CoglFrameEvent event,
-              CoglFrameInfo *info)
-{
-  CoglFrameClosure *entry, *tmp;
-
-  COGL_TAILQ_FOREACH_SAFE (entry,
-                           &onscreen->frame_closures,
-                           list_node,
-                           tmp)
-    {
-      entry->callback (onscreen, event, info,
-                       entry->user_data);
-    }
-}
-
-void
-_cogl_dispatch_onscreen_events (CoglContext *context)
-{
-  CoglOnscreenEvent *event, *tmp;
-  CoglOnscreenEventList queue;
-
-  /* Dispatching the event callback may cause another frame to be
-   * drawn which in may cause another event to be queued immediately.
-   * To make sure this loop will only dispatch one set of events we'll
-   * steal the queue and iterate that separately */
-  COGL_TAILQ_INIT (&queue);
-  COGL_TAILQ_CONCAT (&queue, &context->onscreen_events_queue, list_node);
-  COGL_TAILQ_INIT (&context->onscreen_events_queue);
-
-  COGL_TAILQ_FOREACH_SAFE (event,
-                           &queue,
-                           list_node,
-                           tmp)
-    {
-      CoglOnscreen *onscreen = event->onscreen;
-      CoglFrameInfo *info = event->info;
-
-      notify_event (onscreen, event->type, info);
-
-      cogl_object_unref (onscreen);
-      cogl_object_unref (info);
-
-      g_slice_free (CoglOnscreenEvent, event);
-    }
-}
-
 void
 _cogl_onscreen_notify_frame_sync (CoglOnscreen *onscreen, CoglFrameInfo *info)
 {
@@ -566,17 +612,13 @@ _cogl_onscreen_notify_complete (CoglOnscreen *onscreen, CoglFrameInfo *info)
 void
 _cogl_onscreen_notify_resize (CoglOnscreen *onscreen)
 {
-  CoglResizeNotifyEntry *entry, *tmp;
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
 
-  COGL_TAILQ_FOREACH_SAFE (entry,
-                           &onscreen->resize_callbacks,
-                           list_node,
-                           tmp)
-    entry->callback (onscreen,
-                     framebuffer->width,
-                     framebuffer->height,
-                     entry->user_data);
+  _cogl_closure_list_invoke (&onscreen->resize_closures,
+                             CoglOnscreenResizeCallback,
+                             onscreen,
+                             framebuffer->width,
+                             framebuffer->height);
 }
 
 void
@@ -590,6 +632,10 @@ _cogl_framebuffer_winsys_update_size (CoglFramebuffer *framebuffer,
   framebuffer->height = height;
 
   cogl_framebuffer_set_viewport (framebuffer, 0, 0, width, height);
+
+  if (!(framebuffer->context->private_feature_flags &
+        COGL_PRIVATE_FEATURE_DIRTY_EVENTS))
+    _cogl_onscreen_queue_full_dirty (COGL_ONSCREEN (framebuffer));
 }
 
 void
@@ -620,38 +666,44 @@ cogl_onscreen_get_resizable (CoglOnscreen *onscreen)
   return onscreen->resizable;
 }
 
-unsigned int
-cogl_onscreen_add_resize_handler (CoglOnscreen *onscreen,
-                                  CoglOnscreenResizeCallback callback,
-                                  void *user_data)
+CoglOnscreenResizeClosure *
+cogl_onscreen_add_resize_callback (CoglOnscreen *onscreen,
+                                   CoglOnscreenResizeCallback callback,
+                                   void *user_data,
+                                   CoglUserDataDestroyCallback destroy)
 {
-  CoglResizeNotifyEntry *entry = g_slice_new (CoglResizeNotifyEntry);
-  static int next_resize_callback_id = 0;
-
-  entry->callback = callback;
-  entry->user_data = user_data;
-  entry->id = next_resize_callback_id++;
-
-  COGL_TAILQ_INSERT_TAIL (&onscreen->resize_callbacks, entry, list_node);
-
-  return entry->id;
+  return _cogl_closure_list_add (&onscreen->resize_closures,
+                                 callback,
+                                 user_data,
+                                 destroy);
 }
 
 void
-cogl_onscreen_remove_resize_handler (CoglOnscreen *onscreen,
-                                     unsigned int id)
+cogl_onscreen_remove_resize_callback (CoglOnscreen *onscreen,
+                                      CoglOnscreenResizeClosure *closure)
 {
-  CoglResizeNotifyEntry *entry;
+  _cogl_closure_disconnect (closure);
+}
 
-  COGL_TAILQ_FOREACH (entry, &onscreen->resize_callbacks, list_node)
-    {
-      if (entry->id == id)
-        {
-          COGL_TAILQ_REMOVE (&onscreen->resize_callbacks, entry, list_node);
-          g_slice_free (CoglResizeNotifyEntry, entry);
-          break;
-        }
-    }
+CoglOnscreenDirtyClosure *
+cogl_onscreen_add_dirty_callback (CoglOnscreen *onscreen,
+                                  CoglOnscreenDirtyCallback callback,
+                                  void *user_data,
+                                  CoglUserDataDestroyCallback destroy)
+{
+  return _cogl_closure_list_add (&onscreen->dirty_closures,
+                                 callback,
+                                 user_data,
+                                 destroy);
+}
+
+void
+cogl_onscreen_remove_dirty_callback (CoglOnscreen *onscreen,
+                                     CoglOnscreenDirtyClosure *closure)
+{
+  _COGL_RETURN_IF_FAIL (closure);
+
+  _cogl_closure_disconnect (closure);
 }
 
 int64_t
