@@ -30,6 +30,8 @@
 #include "cogl-winsys-private.h"
 #include "cogl-driver.h"
 #include "cogl-texture-driver.h"
+#include "cogl-context.h"
+#include "cogl-closure-list-private.h"
 
 #ifdef COGL_HAS_XLIB_SUPPORT
 #include <X11/Xlib.h>
@@ -50,6 +52,12 @@ struct _CoglRenderer
   CoglWinsysID winsys_id_override;
   GList *constraints;
 
+  GArray *poll_fds;
+  int poll_fds_age;
+  GList *poll_sources;
+
+  CoglList idle_closures;
+
   GList *outputs;
 
 #ifdef COGL_HAS_XLIB_SUPPORT
@@ -57,6 +65,10 @@ struct _CoglRenderer
   CoglBool xlib_enable_event_retrieval;
 #endif
 
+#ifdef COGL_HAS_WIN32_SUPPORT
+  CoglBool win32_enable_event_retrieval;
+#endif
+
   CoglDriver driver;
 #ifndef HAVE_DIRECTLY_LINKED_GL_LIBRARY
   GModule *libgl_module;
@@ -64,8 +76,7 @@ struct _CoglRenderer
 
 #if defined (COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT)
   struct wl_display *foreign_wayland_display;
-  struct wl_compositor *foreign_wayland_compositor;
-  struct wl_shell *foreign_wayland_shell;
+  CoglBool wayland_enable_event_dispatch;
 #endif
 
 #ifdef COGL_HAS_SDL_SUPPORT
