@@ -40,11 +40,11 @@ typedef struct _CoglTexture CoglTexture;
 #endif
 #include <cogl/cogl-bitmap.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-texture
- * @short_description: Fuctions for creating and manipulating textures
+ * @short_description: Functions for creating and manipulating textures
  *
  * Cogl allows creating and manipulating textures using a uniform
  * API that tries to hide all the various complexities of creating,
@@ -58,7 +58,7 @@ G_BEGIN_DECLS
 /**
  * COGL_TEXTURE_ERROR:
  *
- * #GError domain for texture errors.
+ * #CoglError domain for texture errors.
  *
  * Since: 2.0
  * Stability: Unstable
@@ -100,7 +100,7 @@ typedef enum {
   COGL_TEXTURE_TYPE_RECTANGLE
 } CoglTextureType;
 
-GQuark cogl_texture_error_quark (void);
+uint32_t cogl_texture_error_quark (void);
 
 /**
  * cogl_texture_new_with_size:
@@ -112,7 +112,14 @@ GQuark cogl_texture_error_quark (void);
  *
  * Creates a new #CoglTexture with the specified dimensions and pixel format.
  *
- * Return value: A newly created #CoglTexture or %NULL on failure
+ * The storage for the texture is not necesarily created before this
+ * function returns. The storage can be explicitly allocated using
+ * cogl_texture_allocate() or preferably you can let Cogl
+ * automatically allocate the storage lazily when uploading data when
+ * Cogl may know more about how the texture will be used and can
+ * optimize how it is allocated.
+ *
+ * Return value: A newly created #CoglTexture
  *
  * Since: 0.8
  */
@@ -134,7 +141,7 @@ cogl_texture_new_with_size (unsigned int width,
  *    have non-premultiplied source data and are going to adjust the blend
  *    mode (see cogl_material_set_blend()) or use the data for something
  *    other than straight blending.
- * @error: return location for a #GError or %NULL
+ * @error: return location for a #CoglError or %NULL
  *
  * Creates a #CoglTexture from an image file.
  *
@@ -146,7 +153,7 @@ CoglTexture *
 cogl_texture_new_from_file (const char       *filename,
                             CoglTextureFlags   flags,
                             CoglPixelFormat    internal_format,
-                            GError           **error);
+                            CoglError           **error);
 
 /**
  * cogl_texture_new_from_data:
@@ -173,13 +180,13 @@ cogl_texture_new_from_file (const char       *filename,
  * Since: 0.8
  */
 CoglTexture *
-cogl_texture_new_from_data (unsigned int      width,
-                            unsigned int      height,
-                            CoglTextureFlags  flags,
-                            CoglPixelFormat   format,
-                            CoglPixelFormat   internal_format,
-                            unsigned int      rowstride,
-                            const uint8_t     *data);
+cogl_texture_new_from_data (int width,
+                            int height,
+                            CoglTextureFlags flags,
+                            CoglPixelFormat format,
+                            CoglPixelFormat internal_format,
+                            int rowstride,
+                            const uint8_t *data);
 
 /**
  * cogl_texture_new_from_foreign:
@@ -247,7 +254,7 @@ cogl_is_texture (void *object);
 
 /**
  * cogl_texture_get_width:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  *
  * Queries the width of a cogl texture.
  *
@@ -258,7 +265,7 @@ cogl_texture_get_width (CoglTexture *texture);
 
 /**
  * cogl_texture_get_height:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  *
  * Queries the height of a cogl texture.
  *
@@ -269,7 +276,7 @@ cogl_texture_get_height (CoglTexture *texture);
 
 /**
  * cogl_texture_get_format:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  *
  * Queries the #CoglPixelFormat of a cogl texture.
  *
@@ -308,11 +315,11 @@ cogl_texture_get_format (CoglTexture *texture);
  *                   a mistake that it was ever published.
  */
 unsigned int
-cogl_texture_get_rowstride (CoglTexture *texture);
+cogl_texture_get_rowstride (CoglTexture *texture) COGL_DEPRECATED_IN_1_10;
 
 /**
  * cogl_texture_get_max_waste:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  *
  * Queries the maximum wasted (unused) pixels in one dimension of a GPU side
  * texture.
@@ -324,7 +331,7 @@ cogl_texture_get_max_waste (CoglTexture *texture);
 
 /**
  * cogl_texture_is_sliced:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  *
  * Queries if a texture is sliced (stored as multiple GPU side tecture
  * objects).
@@ -337,7 +344,7 @@ cogl_texture_is_sliced (CoglTexture *texture);
 
 /**
  * cogl_texture_get_gl_texture:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  * @out_gl_handle: (out) (allow-none): pointer to return location for the
  *   textures GL handle, or %NULL.
  * @out_gl_target: (out) (allow-none): pointer to return location for the
@@ -358,7 +365,7 @@ cogl_texture_get_gl_texture (CoglTexture *texture,
 
 /**
  * cogl_texture_get_data:
- * @texture a #CoglTexture pointer.
+ * @texture: a #CoglTexture pointer.
  * @format: the #CoglPixelFormat to store the texture as.
  * @rowstride: the rowstride of @data in bytes or pass 0 to calculate
  *             from the bytes-per-pixel of @format multiplied by the
@@ -383,7 +390,7 @@ cogl_texture_get_data (CoglTexture *texture,
 
 /**
  * cogl_texture_set_region:
- * @texture a #CoglTexture.
+ * @texture: a #CoglTexture.
  * @src_x: upper left coordinate to use from source data.
  * @src_y: upper left coordinate to use from source data.
  * @dst_x: upper left destination horizontal coordinate.
@@ -423,11 +430,69 @@ cogl_texture_set_region (CoglTexture *texture,
 
 #if defined (COGL_ENABLE_EXPERIMENTAL_API)
 
-#define cogl_texture_set_region_from_bitmap \
-  cogl_texture_set_region_from_bitmap_EXP
+/**
+ * cogl_texture_set_data:
+ * @texture a #CoglTexture.
+ * @format: the #CoglPixelFormat used in the source @data buffer.
+ * @rowstride: rowstride of the source @data buffer (computed from
+ *             the texture width and @format if it equals 0)
+ * @data: the source data, pointing to the first top-left pixel to set
+ * @level: The mipmap level to update (Normally 0 for the largest,
+ *         base texture)
+ * @error: A #CoglError to return exceptional errors
+ *
+ * Sets all the pixels for a given mipmap @level by copying the pixel
+ * data pointed to by the @data argument into the given @texture.
+ *
+ * @data should point to the first pixel to copy corresponding
+ * to the top left of the mipmap @level being set.
+ *
+ * If @rowstride equals 0 then it will be automatically calculated
+ * from the width of the mipmap level and the bytes-per-pixel for the
+ * given @format.
+ *
+ * A mipmap @level of 0 corresponds to the largest, base image of a
+ * texture and @level 1 is half the width and height of level 0. If
+ * dividing any dimension of the previous level by two results in a
+ * fraction then round the number down (floor()), but clamp to 1
+ * something like this:
+ *
+ * |[
+ *  next_width = MAX (1, floor (prev_width));
+ * ]|
+ *
+ * You can determine the number of mipmap levels for a given texture
+ * like this:
+ *
+ * |[
+ *  n_levels = 1 + floor (log2 (max_dimension));
+ * ]|
+ *
+ * Where %max_dimension is the larger of cogl_texture_get_width() and
+ * cogl_texture_get_height().
+ *
+ * It is an error to pass a @level number >= the number of levels that
+ * @texture can have according to the above calculation.
+ *
+ * <note>Since the storage for a #CoglTexture is allocated lazily then
+ * if the given @texture has not previously been allocated then this
+ * api can return %FALSE and throw an exceptional @error if there is
+ * not enough memory to allocate storage for @texture.</note>
+ *
+ * Return value: %TRUE if the data upload was successful, and
+ *               %FALSE otherwise
+ */
+CoglBool
+cogl_texture_set_data (CoglTexture *texture,
+                       CoglPixelFormat format,
+                       int rowstride,
+                       const uint8_t *data,
+                       int level,
+                       CoglError **error);
+
 /**
  * cogl_texture_set_region_from_bitmap:
- * @texture a #CoglTexture pointer
+ * @texture: a #CoglTexture pointer
  * @src_x: upper left coordinate to use from the source bitmap.
  * @src_y: upper left coordinate to use from the source bitmap
  * @dst_x: upper left destination horizontal coordinate.
@@ -491,8 +556,6 @@ cogl_texture_new_from_sub_texture (CoglTexture *full_texture,
                                    int sub_width,
                                    int sub_height);
 
-#ifndef COGL_DISABLE_DEPRECATED
-
 /**
  * cogl_texture_ref:
  * @texture: a #CoglTexture.
@@ -504,7 +567,7 @@ cogl_texture_new_from_sub_texture (CoglTexture *full_texture,
  * Return value: the @texture pointer.
  */
 void *
-cogl_texture_ref (void *texture) G_GNUC_DEPRECATED;
+cogl_texture_ref (void *texture) COGL_DEPRECATED_FOR (cogl_object_ref);
 
 /**
  * cogl_texture_unref:
@@ -515,10 +578,30 @@ cogl_texture_ref (void *texture) G_GNUC_DEPRECATED;
  * Deprecated: 1.2: Use cogl_object_unref() instead
  */
 void
-cogl_texture_unref (void *texture) G_GNUC_DEPRECATED;
+cogl_texture_unref (void *texture) COGL_DEPRECATED_FOR (cogl_object_unref);
 
-#endif /* COGL_DISABLE_DEPRECATED */
+/**
+ * cogl_texture_allocate:
+ * @texture: A #CoglTexture
+ * @error: A #CoglError to return exceptional errors or %NULL
+ *
+ * Explicitly allocates the storage for the given @texture which
+ * allows you to be sure that there is enough memory for the
+ * texture and if not then the error can be handled gracefully.
+ *
+ * <note>Normally applications don't need to use this api directly
+ * since the texture will be implicitly allocated when data is set on
+ * the texture, or if the texture is attached to a #CoglOffscreen
+ * framebuffer and rendered too.</note>
+ *
+ * Return value: %TRUE if the texture was successfully allocated,
+ *               otherwise %FALSE and @error will be updated if it
+ *               wasn't %NULL.
+ */
+CoglBool
+cogl_texture_allocate (CoglTexture *texture,
+                       CoglError **error);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_TEXTURE_H__ */
