@@ -44,6 +44,8 @@
  */
 
 #include <cogl/cogl-defines.h>
+#include <cogl/cogl-macros.h>
+
 #include <cogl/cogl-error.h>
 
 #include <cogl/cogl-object.h>
@@ -77,6 +79,7 @@
 #if defined (COGL_ENABLE_EXPERIMENTAL_API)
 #include <cogl/cogl-swap-chain.h>
 #include <cogl/cogl-renderer.h>
+#include <cogl/cogl-output.h>
 #include <cogl/cogl-display.h>
 #include <cogl/cogl-context.h>
 #include <cogl/cogl-buffer.h>
@@ -104,7 +107,9 @@
 #include <cogl/cogl-snippet.h>
 #include <cogl/cogl-framebuffer.h>
 #include <cogl/cogl-onscreen.h>
+#include <cogl/cogl-frame-info.h>
 #include <cogl/cogl-poll.h>
+#include <cogl/cogl-fence.h>
 #if defined (COGL_HAS_EGL_PLATFORM_KMS_SUPPORT)
 #include <cogl/cogl-kms-renderer.h>
 #include <cogl/cogl-kms-display.h>
