@@ -32,6 +32,7 @@
 #include <wayland-client.h>
 #include <wayland-egl.h>
 #include <string.h>
+#include <errno.h>
 
 #include "cogl-winsys-egl-wayland-private.h"
 #include "cogl-winsys-egl-private.h"
@@ -39,6 +40,7 @@
 #include "cogl-onscreen-private.h"
 #include "cogl-wayland-renderer.h"
 #include "cogl-error-private.h"
+#include "cogl-poll-private.h"
 
 static const CoglWinsysEGLVtable _cogl_winsys_egl_vtable;
 
@@ -50,6 +52,7 @@ typedef struct _CoglRendererWayland
   struct wl_compositor *wayland_compositor;
   struct wl_shell *wayland_shell;
   struct wl_registry *wayland_registry;
+  int fd;
 } CoglRendererWayland;
 
 typedef struct _CoglDisplayWayland
@@ -72,6 +75,8 @@ typedef struct _CoglOnscreenWayland
   int pending_dx;
   int pending_dy;
   CoglBool has_pending;
+
+  CoglBool shell_surface_type_set;
 } CoglOnscreenWayland;
 
 static void
@@ -96,8 +101,18 @@ static void
 _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 {
   CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererWayland *wayland_renderer = egl_renderer->platform;
 
-  eglTerminate (egl_renderer->edpy);
+  if (wayland_renderer->wayland_display)
+    {
+      _cogl_poll_renderer_remove_fd (renderer, wayland_renderer->fd);
+
+      if (renderer->foreign_wayland_display == NULL)
+        wl_display_disconnect (wayland_renderer->wayland_display);
+    }
+
+  if (egl_renderer->edpy)
+    eglTerminate (egl_renderer->edpy);
 
   g_slice_free (CoglRendererWayland, egl_renderer->platform);
   g_slice_free (CoglRendererEGL, egl_renderer);
@@ -107,6 +122,58 @@ static const struct wl_registry_listener registry_listener = {
   registry_handle_global_cb,
 };
 
+static int64_t
+prepare_wayland_display_events (void *user_data)
+{
+  CoglRenderer *renderer = user_data;
+  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererWayland *wayland_renderer = egl_renderer->platform;
+  int flush_ret;
+
+  flush_ret = wl_display_flush (wayland_renderer->wayland_display);
+
+  /* If the socket buffer became full then we need to wake up the main
+   * loop once it is writable again */
+  if (flush_ret == -1 && errno == EAGAIN)
+    _cogl_poll_renderer_modify_fd (renderer,
+                                   wayland_renderer->fd,
+                                   COGL_POLL_FD_EVENT_IN |
+                                   COGL_POLL_FD_EVENT_OUT);
+
+  /* Calling this here is a bit dodgy because Cogl usually tries to
+   * say that it won't do any event processing until
+   * cogl_poll_renderer_dispatch is called. However Wayland doesn't
+   * seem to provide any way to query whether the event queue is empty
+   * and we would need to do that in order to force the main loop to
+   * wake up to call it from dispatch. */
+  wl_display_dispatch_pending (wayland_renderer->wayland_display);
+
+  return -1;
+}
+
+static void
+dispatch_wayland_display_events (void *user_data, int revents)
+{
+  CoglRenderer *renderer = user_data;
+  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererWayland *wayland_renderer = egl_renderer->platform;
+
+  if ((revents & COGL_POLL_FD_EVENT_IN))
+    wl_display_dispatch (wayland_renderer->wayland_display);
+
+  if ((revents & COGL_POLL_FD_EVENT_OUT))
+    {
+      int ret = wl_display_flush (wayland_renderer->wayland_display);
+
+      if (ret != -1 || errno != EAGAIN)
+        /* There is no more data to write so we don't need to wake up
+         * when the write buffer is emptied anymore */
+        _cogl_poll_renderer_modify_fd (renderer,
+                                       wayland_renderer->fd,
+                                       COGL_POLL_FD_EVENT_IN);
+    }
+}
+
 static CoglBool
 _cogl_winsys_renderer_connect (CoglRenderer *renderer,
                                CoglError **error)
@@ -130,15 +197,6 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
   if (renderer->foreign_wayland_display)
     {
       wayland_renderer->wayland_display = renderer->foreign_wayland_display;
-      /* XXX: For now we have to assume that if a foreign display is
-       * given then a foreign compositor and shell must also have been
-       * given because wayland doesn't provide a way to
-       * retrospectively be notified of the these objects. */
-      g_assert (renderer->foreign_wayland_compositor);
-      g_assert (renderer->foreign_wayland_shell);
-      wayland_renderer->wayland_compositor =
-        renderer->foreign_wayland_compositor;
-      wayland_renderer->wayland_shell = renderer->foreign_wayland_shell;
     }
   else
     {
@@ -150,15 +208,15 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
                        "Failed to connect wayland display");
           goto error;
         }
-
-      wayland_renderer->wayland_registry =
-        wl_display_get_registry (wayland_renderer->wayland_display);
-
-      wl_registry_add_listener (wayland_renderer->wayland_registry,
-                                &registry_listener,
-                                egl_renderer);
     }
 
+  wayland_renderer->wayland_registry =
+    wl_display_get_registry (wayland_renderer->wayland_display);
+
+  wl_registry_add_listener (wayland_renderer->wayland_registry,
+                            &registry_listener,
+                            egl_renderer);
+
   /*
    * Ensure that that we've received the messages setting up the
    * compostor and shell object. This is better than just
@@ -175,6 +233,16 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
   if (!_cogl_winsys_egl_renderer_connect_common (renderer, error))
     goto error;
 
+  wayland_renderer->fd = wl_display_get_fd (wayland_renderer->wayland_display);
+
+  if (renderer->wayland_enable_event_dispatch)
+    _cogl_poll_renderer_add_fd (renderer,
+                                wayland_renderer->fd,
+                                COGL_POLL_FD_EVENT_IN,
+                                prepare_wayland_display_events,
+                                dispatch_wayland_display_events,
+                                renderer);
+
   return TRUE;
 
 error:
@@ -300,6 +368,16 @@ _cogl_winsys_egl_context_init (CoglContext *context,
                   COGL_WINSYS_FEATURE_MULTIPLE_ONSCREEN,
                   TRUE);
 
+  /* We'll manually handle queueing dirty events when the surface is
+   * first shown or when it is resized. Note that this is slightly
+   * different from the emulated behaviour that CoglFramebuffer would
+   * provide if we didn't set this flag because we want to emit the
+   * event on show instead of on allocation. The Wayland protocol
+   * delays setting the surface type until the next buffer is attached
+   * so attaching a buffer before setting the type would not cause
+   * anything to be displayed */
+  context->private_feature_flags |= COGL_PRIVATE_FEATURE_DIRTY_EVENTS;
+
   return TRUE;
 }
 
@@ -319,8 +397,12 @@ _cogl_winsys_egl_onscreen_init (CoglOnscreen *onscreen,
   wayland_onscreen = g_slice_new0 (CoglOnscreenWayland);
   egl_onscreen->platform = wayland_onscreen;
 
-  wayland_onscreen->wayland_surface =
-    wl_compositor_create_surface (wayland_renderer->wayland_compositor);
+  if (onscreen->foreign_surface)
+    wayland_onscreen->wayland_surface = onscreen->foreign_surface;
+  else
+    wayland_onscreen->wayland_surface =
+      wl_compositor_create_surface (wayland_renderer->wayland_compositor);
+
   if (!wayland_onscreen->wayland_surface)
     {
       _cogl_set_error (error, COGL_WINSYS_ERROR,
@@ -329,10 +411,6 @@ _cogl_winsys_egl_onscreen_init (CoglOnscreen *onscreen,
       return FALSE;
     }
 
-  wayland_onscreen->wayland_shell_surface =
-    wl_shell_get_shell_surface (wayland_renderer->wayland_shell,
-                                wayland_onscreen->wayland_surface);
-
   wayland_onscreen->wayland_egl_native_window =
     wl_egl_window_create (wayland_onscreen->wayland_surface,
                           cogl_framebuffer_get_width (framebuffer),
@@ -353,7 +431,10 @@ _cogl_winsys_egl_onscreen_init (CoglOnscreen *onscreen,
                             wayland_onscreen->wayland_egl_native_window,
                             NULL);
 
-  wl_shell_surface_set_toplevel (wayland_onscreen->wayland_shell_surface);
+  if (!onscreen->foreign_surface)
+    wayland_onscreen->wayland_shell_surface =
+      wl_shell_get_shell_surface (wayland_renderer->wayland_shell,
+                                  wayland_onscreen->wayland_surface);
 
   return TRUE;
 }
@@ -370,23 +451,30 @@ _cogl_winsys_egl_onscreen_deinit (CoglOnscreen *onscreen)
       wayland_onscreen->wayland_egl_native_window = NULL;
     }
 
-  if (wayland_onscreen->wayland_surface)
+  if (!onscreen->foreign_surface)
     {
-      wl_surface_destroy (wayland_onscreen->wayland_surface);
-      wayland_onscreen->wayland_surface = NULL;
+      /* NB: The wayland protocol docs explicitly state that
+       * "wl_shell_surface_destroy() must be called before destroying
+       * the wl_surface object." ... */
+      if (wayland_onscreen->wayland_shell_surface)
+        {
+          wl_shell_surface_destroy (wayland_onscreen->wayland_shell_surface);
+          wayland_onscreen->wayland_shell_surface = NULL;
+        }
+
+      if (wayland_onscreen->wayland_surface)
+        {
+          wl_surface_destroy (wayland_onscreen->wayland_surface);
+          wayland_onscreen->wayland_surface = NULL;
+        }
     }
 
   g_slice_free (CoglOnscreenWayland, wayland_onscreen);
 }
 
 static void
-_cogl_winsys_onscreen_swap_buffers (CoglOnscreen *onscreen)
+flush_pending_resize (CoglOnscreen *onscreen)
 {
-  CoglFramebuffer *fb = COGL_FRAMEBUFFER (onscreen);
-  CoglContext *context = fb->context;
-  CoglRenderer *renderer = context->display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
-  CoglRendererWayland *wayland_renderer = egl_renderer->platform;
   CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
   CoglOnscreenWayland *wayland_onscreen = egl_onscreen->platform;
 
@@ -398,22 +486,55 @@ _cogl_winsys_onscreen_swap_buffers (CoglOnscreen *onscreen)
                             wayland_onscreen->pending_dx,
                             wayland_onscreen->pending_dy);
 
-      _cogl_framebuffer_winsys_update_size (fb,
+      _cogl_framebuffer_winsys_update_size (COGL_FRAMEBUFFER (onscreen),
                                             wayland_onscreen->pending_width,
                                             wayland_onscreen->pending_height);
+
+      _cogl_onscreen_queue_full_dirty (onscreen);
+
+      wayland_onscreen->pending_dx = 0;
+      wayland_onscreen->pending_dy = 0;
       wayland_onscreen->has_pending = FALSE;
     }
+}
 
-  /* chain-up */
-  parent_vtable->onscreen_swap_buffers (onscreen);
+static void
+_cogl_winsys_onscreen_swap_buffers_with_damage (CoglOnscreen *onscreen,
+                                                const int *rectangles,
+                                                int n_rectangles)
+{
+  flush_pending_resize (onscreen);
 
-  /*
-   * The implementation of eglSwapBuffers may do a flush however the semantics
-   * of eglSwapBuffers on Wayland has changed in the past. So to be safe to
-   * the implementation changing we should explicitly ensure all messages are
-   * sent.
-   */
-  wl_display_flush (wayland_renderer->wayland_display);
+  parent_vtable->onscreen_swap_buffers_with_damage (onscreen,
+                                                    rectangles,
+                                                    n_rectangles);
+}
+
+static void
+_cogl_winsys_onscreen_set_visibility (CoglOnscreen *onscreen,
+                                      CoglBool visibility)
+{
+  CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
+  CoglOnscreenWayland *wayland_onscreen = egl_onscreen->platform;
+
+  /* The first time the onscreen is shown we will set it to toplevel
+   * so that it will appear on the screen. If the surface is foreign
+   * then we won't have the shell surface and we'll just let the
+   * application deal with setting the surface type. */
+  if (visibility &&
+      wayland_onscreen->wayland_shell_surface &&
+      !wayland_onscreen->shell_surface_type_set)
+    {
+      wl_shell_surface_set_toplevel (wayland_onscreen->wayland_shell_surface);
+      wayland_onscreen->shell_surface_type_set = TRUE;
+      _cogl_onscreen_queue_full_dirty (onscreen);
+    }
+
+  /* FIXME: We should also do something here to hide the surface when
+   * visilibity == FALSE. It sounds like there are currently ongoing
+   * discussions about adding support for hiding surfaces in the
+   * Wayland protocol so we might as well wait until then to add that
+   * here. */
 }
 
 void
@@ -428,6 +549,17 @@ cogl_wayland_renderer_set_foreign_display (CoglRenderer *renderer,
   renderer->foreign_wayland_display = display;
 }
 
+void
+cogl_wayland_renderer_set_event_dispatch_enabled (CoglRenderer *renderer,
+                                                  CoglBool enable)
+{
+  _COGL_RETURN_IF_FAIL (cogl_is_renderer (renderer));
+  /* NB: Renderers are considered immutable once connected */
+  _COGL_RETURN_IF_FAIL (!renderer->connected);
+
+  renderer->wayland_enable_event_dispatch = enable;
+}
+
 struct wl_display *
 cogl_wayland_renderer_get_display (CoglRenderer *renderer)
 {
@@ -445,94 +577,44 @@ cogl_wayland_renderer_get_display (CoglRenderer *renderer)
     return NULL;
 }
 
-void
-cogl_wayland_renderer_set_foreign_compositor (CoglRenderer *renderer,
-                                              struct wl_compositor *compositor)
-{
-  _COGL_RETURN_IF_FAIL (cogl_is_renderer (renderer));
-
-  /* NB: Renderers are considered immutable once connected */
-  _COGL_RETURN_IF_FAIL (!renderer->connected);
-
-  renderer->foreign_wayland_compositor = compositor;
-}
-
-struct wl_compositor *
-cogl_wayland_renderer_get_compositor (CoglRenderer *renderer)
-{
-  _COGL_RETURN_VAL_IF_FAIL (cogl_is_renderer (renderer), NULL);
-
-  if (renderer->foreign_wayland_compositor)
-    return renderer->foreign_wayland_compositor;
-  else if (renderer->connected)
-    {
-      CoglRendererEGL *egl_renderer = renderer->winsys;
-      CoglRendererWayland *wayland_renderer = egl_renderer->platform;
-      return wayland_renderer->wayland_compositor;
-    }
-  else
-    return NULL;
-}
-
-void
-cogl_wayland_renderer_set_foreign_shell (CoglRenderer *renderer,
-                                         struct wl_shell *shell)
-{
-  _COGL_RETURN_IF_FAIL (cogl_is_renderer (renderer));
-
-  /* NB: Renderers are considered immutable once connected */
-  _COGL_RETURN_IF_FAIL (!renderer->connected);
-
-  renderer->foreign_wayland_shell = shell;
-}
-
-struct wl_shell *
-cogl_wayland_renderer_get_shell (CoglRenderer *renderer)
-{
-  _COGL_RETURN_VAL_IF_FAIL (cogl_is_renderer (renderer), NULL);
-
-  if (renderer->foreign_wayland_shell)
-    return renderer->foreign_wayland_shell;
-  else if (renderer->connected)
-    {
-      CoglRendererEGL *egl_renderer = renderer->winsys;
-      CoglRendererWayland *wayland_renderer = egl_renderer->platform;
-      return wayland_renderer->wayland_shell;
-    }
-  else
-    return NULL;
-}
-
 struct wl_surface *
 cogl_wayland_onscreen_get_surface (CoglOnscreen *onscreen)
 {
-  CoglFramebuffer *fb;
+  CoglOnscreenEGL *egl_onscreen;
+  CoglOnscreenWayland *wayland_onscreen;
 
-  fb = COGL_FRAMEBUFFER (onscreen);
-  if (fb->allocated)
-    {
-      CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-      CoglOnscreenWayland *wayland_onscreen = egl_onscreen->platform;
-      return wayland_onscreen->wayland_surface;
-    }
-  else
-    return NULL;
+  cogl_framebuffer_allocate (COGL_FRAMEBUFFER (onscreen), NULL);
+
+  egl_onscreen = onscreen->winsys;
+  wayland_onscreen = egl_onscreen->platform;
+
+  return wayland_onscreen->wayland_surface;
 }
 
 struct wl_shell_surface *
 cogl_wayland_onscreen_get_shell_surface (CoglOnscreen *onscreen)
+{
+  CoglOnscreenEGL *egl_onscreen;
+  CoglOnscreenWayland *wayland_onscreen;
+
+  cogl_framebuffer_allocate (COGL_FRAMEBUFFER (onscreen), NULL);
+
+  egl_onscreen = onscreen->winsys;
+  wayland_onscreen = egl_onscreen->platform;
+
+  return wayland_onscreen->wayland_shell_surface;
+}
+
+void
+cogl_wayland_onscreen_set_foreign_surface (CoglOnscreen *onscreen,
+                                           struct wl_surface *surface)
 {
   CoglFramebuffer *fb;
 
   fb = COGL_FRAMEBUFFER (onscreen);
-  if (fb->allocated)
-    {
-      CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-      CoglOnscreenWayland *wayland_onscreen = egl_onscreen->platform;
-      return wayland_onscreen->wayland_shell_surface;
-    }
-  else
-    return NULL;
+  _COGL_RETURN_IF_FAIL (!fb->allocated);
+
+  onscreen->foreign_surface = surface;
 }
 
 void
@@ -560,6 +642,15 @@ cogl_wayland_onscreen_resize (CoglOnscreen *onscreen,
           wayland_onscreen->pending_dx += offset_x;
           wayland_onscreen->pending_dy += offset_y;
           wayland_onscreen->has_pending = TRUE;
+
+          /* If nothing has been drawn to the framebuffer since the
+           * last swap then wl_egl_window_resize will take effect
+           * immediately. Otherwise it might not take effect until the
+           * next swap, depending on the version of Mesa. To keep
+           * consistent behaviour we'll delay the resize until the
+           * next swap unless we're sure nothing has been drawn */
+          if (!fb->mid_scene)
+            flush_pending_resize (onscreen);
         }
     }
   else
@@ -598,7 +689,11 @@ _cogl_winsys_egl_wayland_get_vtable (void)
       vtable.renderer_connect = _cogl_winsys_renderer_connect;
       vtable.renderer_disconnect = _cogl_winsys_renderer_disconnect;
 
-      vtable.onscreen_swap_buffers = _cogl_winsys_onscreen_swap_buffers;
+      vtable.onscreen_swap_buffers_with_damage =
+        _cogl_winsys_onscreen_swap_buffers_with_damage;
+
+      vtable.onscreen_set_visibility =
+        _cogl_winsys_onscreen_set_visibility;
 
       vtable_inited = TRUE;
     }
